'*******************************************************************************
'*                             TControl.bi                                     *
'*             This file is a part of sGUI libary for FreeBasic                *
'*             Copyright 2010-2013 Nastasa Vasile Eodor                       *
'*             http://sgui.rqwork.evonet.ro                                    *
'*             mail: nastase.eodor@gmail.com                                   *
'*                             version 2.0                                     *
'*******************************************************************************

#include once "windows.bi"
#include once "win/commctrl.bi"

#include once "sysutils.bi"

#ifndef TMessageClass
#define TMessageClass(__Ptr__) *cast(TControl ptr,__Ptr__)
#endif

#ifndef TControlClass
#define TControlClass(__Ptr__) *cast(TControl ptr,__Ptr__)
#endif

#ifndef GetControl
#define GetControl(__HWND__) *cast(TControl Ptr,GetWindowLong(__HWND__,GetClassLong(__HWND__,gcl_cbwndextra)-4))
#endif

type PDesigner as TDesigner ptr
type PControl as TControl ptr
type PMessage as TMessage ptr
type PTimer as TTimer ptr
    
enum TControlStyle
    csDefault=0,csNoInstance,csAcceptControls
end enum

type TMessage
    Handle  as hwnd
    uid     as uint
    wparam  as wparam
    lparam  as lparam
    result  as lresult
    Control as PControl
end type

type TTimer
    private:
    FParent   as PControl
    FInterval as integer
    FEnabled  as integer
    public:
    Id        as integer
    declare property Parent as PControl
    declare property Parent(value as PControl)
    declare property Iterval as integer
    declare property Interval(value as integer)
    declare property Enabled as integer
    declare property Enabled(value as integer)
    declare constructor(AParent as PControl=0,AId as integer=1,AInterval as integer=1000,AEnabled as integer=0)
    declare destructor
    declare operator cast as any ptr
    declare operator cast as string
    onTimer as sub(byref Sender as TTimer)
end type

type TControl
    private:
    FDesignMode     as integer
    FDoubleBuffered as integer
    FClassName      as string
    FClassAncestor  as string
    FExStyle        as integer
    FStyle          as integer
    FText           as string
    FAlign          as integer
    FLeft           as integer
    FTop            as integer
    FWidth          as integer
    FHeight         as integer
    FClientWidth    as integer
    FClientHeight   as integer
    FParentWindow   as hwnd
    FParent         as PControl
    FParentBrush    as integer
    FParentColor    as integer
    FParentFont     as integer
    FActiveControl  as PControl
    FColor          as integer
    FBrush          as hbrush
    FTabstop        as integer
    FTaborder       as integer
    FEnabled        as integer
    FVisible        as integer
    FHint           as string
    FShowHint       as integer
    FZPosition      as integer
    FHandle         as hwnd 
    FToolHandle     as hwnd
    FToolInfo       as toolinfo
    FControlCount   as integer
    FControls       as PControl ptr
    FTimerCount     as integer
    FTimers         as TTimer ptr ptr
    declare sub AllocateHint
    declare sub CreateHandle
    declare sub Add(value as PControl)
    declare sub Insert(Index as integer,value as PControl)
    declare sub Remove(value as PControl)
    public:
    Designer        as PDesigner
    ControlStyle    as integer
    Name            as string 
    ID              as integer
    Captured        as PControl
    Tag             as any ptr
    declare constructor(byref AClassName as string="",byref AClassAncestor as string="",byref AText as string="",ALeft as integer=0,ATop as integer=0,AWidth as integer=0,AHeight as integer=0,AStyle as integer=0,AExStyle as integer=0,AParent as TControl ptr=0,Proc as WndProc = 0)
    declare constructor(AHandle as hwnd)
    declare destructor
    declare operator cast as any ptr
    declare operator cast as string
    declare property ClassName as string
    declare property ClassName(value as string)
    declare property ClassAncestor as string
    declare property ClassAncestor(value as string)
    declare property ExStyle as integer
    declare property ExStyle(value as integer)
    declare property Style as integer
    declare property Style(value as integer)
    declare property Text as string
    declare property Text(value as string)
    declare property Align as integer
    declare property Align(value as integer)
    declare property Left as integer
    declare property Left(value as integer)
    declare property Top as integer
    declare property Top(value as integer)
    declare property Width as integer
    declare property Width(value as integer)
    declare property Height as integer
    declare property Height(value as integer)
    declare property ClientWidth as integer
    declare property ClientWidth(value as integer)
    declare property ClientHeight as integer
    declare property ClientHeight(value as integer)
    declare property Enabled as integer
    declare property Enabled(value as integer)
    declare property Visible as integer
    declare property Visible(value as integer)
    declare property Color as integer
    declare property Color(value as integer)
    declare property Brush as hbrush
    declare property Brush(value as hbrush)
    declare property Hint As String
    declare property Hint(value As String)
    declare property ShowHint As Integer
    declare property ShowHint(value As Integer)
    declare property ParentWindow as hwnd
    declare property ParentWindow(value as hwnd)
    declare property ParentFont as integer
    declare property ParentFont(value as integer)
    declare property ParentColor as integer
    declare property ParentColor(value as integer)
    declare property ParentBrush as integer
    declare property ParentBrush(value as integer)
    declare property ActiveControl as PControl
    declare property ActiveControl(value as PControl)
    declare property Parent as PControl
    declare property Parent(value as PControl)
    declare property ToolHandle as hwnd
    declare property ToolHandle(value as hwnd)
    declare property Handle as hwnd
    declare property Handle(value as hwnd)
    declare property ControlCount as integer
    declare property ControlCount(value as integer)
    declare property Control(index as integer) as PControl
    declare property Control(index as integer,value as PControl)
    declare property DoubleBuffered as integer
    declare property DoubleBuffered(value as integer)
    declare property DesignMode as integer
    declare property DesignMode(value as integer)
    declare property TabStop as integer
    declare property TabStop(value as integer)
    declare property TabOrder as integer
    declare property TabOrder(value as integer)
    declare property TimerCount as integer
    declare property TimerCount(value as integer)
    declare property Timer(index as integer) as TTimer ptr
    declare property Timer(index as integer,value as TTimer ptr)
    declare function IndexOf(value as PControl) as integer
    declare function SelectNext(CurControl as TControl ptr) as TControl ptr
    declare sub SetBounds(R as Rect)
    declare sub RequestAlign
    declare sub RecreateHandle
    declare sub FreeHandle
    declare sub AddTimer overload(AId as integer,AInterval as integer=1000,AEnabled as integer=0)
    declare sub AddTimer(ATimer as TTimer ptr)
    declare sub RemoveTimer overload(AId as integer)
    declare sub RemoveTimer(ATimer as TTimer ptr)
    declare sub ClientToScreen(byref P as Point)
    declare sub ScreenToClient(byref P as Point)
    declare sub Repaint
    declare sub Invalidate
    declare sub Update
    declare sub BringToFront
    declare sub SendToBack
    declare sub SetFocus
    onWindowProc as sub(byref message as TMessage)
end type

declare function IsDesignMessage(uid as uint) as integer


const EM_GETSELTEXT      = (wm_user + 62)

const appCreate          = wm_app + &h1
const appCommand         = wm_app + &h2
const appNotify          = wm_app + &h3
const appColor           = wm_app + &h4
const appHScroll         = wm_app + &h5
const appVScroll         = wm_app + &h6
const appDrawItem        = wm_app + &h7
const appMeasureItem     = wm_app + &h8
const appDesignTimeMouse = wm_app + &h9
const appDesignTimeKey   = wm_app + &h10

const mdi_clientid       = &HCAC
const mdi_firstchild     = &HFF00

common shared as PControl CreationData
common shared as integer AllowRunTimeCreation 
